import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.Writable;
import org.apache.mahout.classifier.naivebayes.NaiveBayesModel;
import org.apache.mahout.classifier.naivebayes.StandardNaiveBayesClassifier;
import org.apache.mahout.common.Pair;
import org.apache.mahout.common.iterator.sequencefile.SequenceFileIterator;
import org.apache.mahout.math.RandomAccessSparseVector;
import org.apache.mahout.math.Vector;


public class ClassificationNB {

	private String modelFilePath;
	private String classifiedFilePath;
	private String dictionaryFilePath;
	
	/*
	 * main
	 */
	public static void main(String[] args) {
		ClassificationNB classifier = new ClassificationNB();
		classifier.execute(args);
	}
	
	
	/*
	 * 実処理
	 * 
	 * args[0] modelファイルへのパス
	 * args[1] 分類対象ファイルへのパス
	 * args[2] 辞書ファイルへのパス
	 */
	private void execute(String[] args){
		//引数解析
		parseArgs(args);
		
		//分類
		classify();
	}
	
	
	/*
	 * 引数解析
	 */
	private void parseArgs(String[] args){
		//modelファイル
		modelFilePath = args[0];
		//分類対象ファイル
		classifiedFilePath = args[1];
		//単語とIDのひも付け取得用
		dictionaryFilePath = args[2];
	}
	
	
	/*
	 * 分類処理
	 */
	private void classify(){
		Configuration conf = new Configuration();

		NaiveBayesModel model = null;
		try{
			//モデル読み込み
			model = NaiveBayesModel.materialize(new Path(modelFilePath), conf);
		}catch(IOException ex){
			ex.printStackTrace();
			System.exit(1);
		}
		StandardNaiveBayesClassifier classifier = new StandardNaiveBayesClassifier(model);
		
		//単語とIDのひもづき取得
		SequenceFileIterator<?, ?> dicIterator = null;
		try{
			dicIterator = new SequenceFileIterator<Writable, Writable>(new Path(dictionaryFilePath), true, conf);
		}catch(IOException ex){
			ex.printStackTrace();
			System.exit(1);
		}
		HashMap<String, Integer> wordIDMap = new HashMap<String, Integer>();
		while (dicIterator.hasNext()) {
			Pair<?, ?> record = dicIterator.next();
			String key = record.getFirst().toString();
			Integer val = Integer.valueOf(record.getSecond().toString());
			wordIDMap.put(key, val);
		}
		
		//出現する単語数をベクトルの次元数とする
		int wordNum = wordIDMap.size();
		
		//ベクトル生成
		Vector vector = new RandomAccessSparseVector(wordNum);
		
		//分類対象読み込み
		List<String> targetDocWordList = loadCFFile();
		
		//とりあえず重みは出現回数とする
		for(String word : targetDocWordList){ //targetDocWordListは分類対象に含まれているすべての単語を格納
		    if(wordIDMap.get(word) == null){
		        continue;
		    }
		    int wordID = wordIDMap.get(word);
		    vector.setQuick(wordID, vector.get(wordID) + 1);
		}

		//分類実行
		Vector result = classifier.classifyFull(vector);

		//分類結果出力
		for (int i = 0; i < result.size(); i++){
			System.out.println("label id:"+i+" probability:"+result.get(i));
		}

	}
	
	
	/*
	 * 分かち書き済みファイルを単語のリストとしてロード
	 */
	private List<String> loadCFFile(){
		BufferedReader bf = null;
		String readline = "";
		String fileContent = "";

		try {
			bf = new BufferedReader(new FileReader(new File(classifiedFilePath)));
			while ((readline = bf.readLine()) != null) {
				fileContent += " ";
				fileContent += readline;
			}
			bf.close();
		} catch (Exception ex) {
			ex.printStackTrace();
			System.exit(1);
		}
		
		return Arrays.asList(fileContent.split("[ ]+"));
	}
	
}
